/* global process */

'use strict';

var
    util    = require('util'),
    Taker   = require('undertaker'),
    watch   = require('glob-watcher'),
    logger  = require('gulplog'),
    
    bundle  = require('./lib/bundle'),
    tasks   = require('./lib/tasks'),
    input   = require('./lib/input'),
    output  = require('./lib/output'),
    Config  = require('./lib/config'),
    
    boot    = require('./lib/bundle/loader')
;

var Huddler = function ()
{
    if (!(this instanceof Huddler)) {
        return new Huddler();
    }
    
    Taker.call(this);
    
    var configPath = 'huddler.json';
    var index = process.argv.indexOf('--config');
    if (-1 !== index) {
        configPath = process.argv[index + 1];
    }
    
    this._options = new Config(configPath);

    this.assets = input(this._options);
    this.webDir = output(this._options);
    
    this.series = this.series.bind(this);
    this.parallel = this.parallel.bind(this);
    
    this.task = (name, fn) => {
        logger.debug('Registering task "' + name + '"');
        return Taker.prototype.task.call(this, name, tasks.track(name, fn));
    };
    
    if ('parallel' === this._options.get('merge')) {
        this.huddle = bundle.mergeParallel.bind(this);
    } else {
        this.huddle = bundle.mergeSeries.bind(this);
    }
};

util.inherits(Huddler, Taker);

Huddler.prototype.getOption = function () {
    return Config.prototype.get.apply(this._options, arguments);
};

Huddler.prototype.watch = function (glob, opt, task) {
    if (typeof opt === 'string' || typeof task === 'string' || Array.isArray(opt) || Array.isArray(task)) {
        throw new Error(
            'watching ' + glob + ': watch task has to be ' +
            'a function (optionally generated by using gulp.parallel ' +
            'or gulp.series)'
        );
    }

    if (typeof opt === 'function') {
        task = opt;
        opt = {};
    }

    opt = opt || {};

    var fn;
    if (typeof task === 'function') {
        fn = this.parallel(task);
    }

    return watch(glob, opt, fn);
};

var inst = new Huddler();

// Setup the exports before loading the bundles to avoid
// problems with circular references.
module.exports = inst;

// Load the bundles
boot(inst);