'use strict';

function BundleTask(name, callback) {
    this.name = name;
    this.result = null;
    this.complete = false;
    this.running = false;
    this.completion_handlers = [];
    this.callback = callback;
};

BundleTask.prototype.register = function(taker, name) {
    let self = this;
    var wrapper = function(cb) {
        return self._handleComplete(self.callback.apply({name: name}, (result) => {
            self._handleComplete(result, cb);
        }));
    };
    
    return taker(name, wrapper);
};

BundleTask.prototype.onCompletion = function(callback) {
    if (this.complete) {
        callback(this.result);
        return;
    }
    
    this.completion_handlers.push(callback);
};

BundleTask.prototype._handleComplete = function(result, callback) {
    if (result && !this.complete) {
        this.complete = true;
        this.result = result;
        
        if (callback) {
            callback();
        }
        
        while (this.completion_handlers.length) {
            let cb = this.completion_handlers.shift();
            cb(this.result);
        }
    }
    
    return result;
};

module.exports = BundleTask;