'use strict';

var
    PassThrough = require('stream').PassThrough,
    unique      = require('unique-stream'),
    pumpify     = require('pumpify'),
    
    cache       = require('./cache'),
    filter      = require('./filter'),
    currentTask = require('../tasks').current
;

function mergeParallel(bundles, options)
{
    var task = currentTask();
    
    /*
     * When a stream becomes available from a bundled task, pipe it to the
     * merged stream. When all streams have ended, the merged stream should
     * also end. The order of ends is not important.
     */
    bundles = cache[task] ? filter(cache[task], bundles) : new Map();
    options = options || {};
    
    options.end = false;
    options.objectMode = options.objectMode || true;
    options.highWaterMark = options.highWaterMark || 64 * 1024;
    
    var target = PassThrough(options);
    var pipes = bundles.size;
    
    if (pipes === 0) {
        target.end();
        return target;
    }
    
    function onEnd () {
        target.setMaxListeners(target.getMaxListeners() - 1);
        if (--pipes <= 0) {
            target.end();
        }
    };
    
    for (var task of bundles.values()) {
        task.onCompletion((result) => {
            // If result is a stream and not ended, then pipe to target
            if (result._readableState && !result._readableState.endEmitted && result.pause && result.pipe) {
                target.setMaxListeners(target.getMaxListeners() + 1);
                result.pipe(target, {end: false});
                result.once('end', onEnd);
            } else if (--pipes <= 0) {
                target.end();
            }
        });
    };
    
    // start each task
    this.parallel(Array.from(bundles.keys()))((error) => {
        if (error) {
            target.emit('error', error);
        }
    });

    return pumpify.obj(target, unique('path'));
};

module.exports = mergeParallel;