'use strict';

var
    Stream      = require('merge2'),
    unique      = require('unique-stream'),
    pumpify     = require('pumpify'),
    
    cache       = require('./cache'),
    filter      = require('./filter')
;

const __NAME__   = 0,
      __STREAM__ = 1;

function mergeSeries(bundles, options)
{
    var task = currentTask();
    
    /*
     * When a stream becomes available from the first bundled task, pipe it
     * to the merged stream. When the stream has ended, move onto the next
     * bundle. When all bundles complete, end the merged stream.
     */
    bundles = cache[task] ? filter(cache[task], bundles) : new Map();
    options = options || {};
    
    options.end = false;
    options.objectMode = options.objectMode || true;
    options.highWaterMark = options.highWaterMark || 64 * 1024;
    
    var target = Stream(options);
    var candidates = new Map();
    var pipes = bundles.size;
    
    if (pipes === 0) {
        target.end();
        return target;
    }
    
    function onEnd () {
        if (--pipes <= 0) {
            target.end();
        }
    };
    
    function addStream(name, stream) {
        // If result is a stream, merge
        if (stream._readableState && stream.pause && stream.pipe) {
            candidates.set(name, stream);
            stream.once('end', onEnd);
        } else {
            // Remove from candidates
            candidates.delete(name);
        }
    };
    
    function nextStream() {
        for (var entry of candidates.entries()) {
            if (entry[__STREAM__]) {
                candidates.delete(entry[__NAME__]);
            }
            return entry[__STREAM__];
        }
        return false;
    };
    
    for (var name of bundles.keys()) {
        candidates.set(name, false);
        
        bundles.get(name).onCompletion((result) => {
            addStream(name, result);
        
            let stream = nextStream();
            
            while (false !== stream) {
                target.add(stream);
                stream = nextStream();
            }
        });
    };
    
    // start each task
    this.parallel(Array.from(candidates.keys()))((error) => {
        if (error) {
            target.emit('error', error);
        }
    });

    return pumpify.obj(target, unique('path'));
};

module.exports = mergeSeries;