'use strict';

var
    vinyl = require('vinyl-fs'),
    through = require('through2'),
    sourcemap = require('vinyl-sourcemap'),
    pumpify = require('pumpify'),
    lead = require('lead'),
    
    hash = require('../util/hasher')
;

function hashContent() {
    return through.obj((file, enc, cb) => {
        if (file.isBuffer()) {
            file.hash = hash(file);
        }
        
        cb(null, file);
    });
};

function addSourcemaps(target) {
    return through.obj((file, enc, cb) => {
        if (!target) {
            return cb(null, file);
        }

        sourcemap.add(file, (err, updatedFile) => {
            if (err) {
                return callback(err);
            }

            cb(null, updatedFile);
        });
    });
};

module.exports = function(options)
{
    return function(glob, opt)
    {
        var target = opt = options.resolve(opt).get('sourcemaps');

        var loadStream = pumpify.obj([
            // current gulp v4-alpha-2 uses gulp-sourcemaps, we use vinyl-sourcemap
            vinyl.src(glob, Object.assign({}, opt, {sourcemaps: false})),
            hashContent(),
            addSourcemaps(target)
        ]);

        return lead(loadStream);
    };
};