'use strict';

var
    less       = require('gulp-less'),
    sass       = require('gulp-sass'),
    concat     = require('gulp-concat'),
    prefix     = require('gulp-autoprefixer'),
    cleancss   = require('gulp-clean-css'),
    coffee     = require('gulp-coffee'),
    coffeelint = require('gulp-coffeelint'),
    babel      = require('gulp-babel'),
    jshint     = require('gulp-jshint'),
    uglify     = require('gulp-uglify'),
    merge      = require('merge2')
;

module.exports = function (symfony, bundle_name, bundle_path) {
    var asset_path   = bundle_path + '/Resources/public/';
    
    function css_stream(opt) {
        return symfony
            .assets(asset_path + '/css/*.css', opt)
        ;
    };
    
    function less_stream(opt) {
        return symfony
            .assets(asset_path + '/less/*.less', opt)
                .pipe(less())
        ;
    };
    
    function sass_stream(opt) {
        return symfony
            .assets(asset_path + '/sass/*.sass', opt)
                .pipe(sass())
        ;
    };
    
    function js_stream(opt) {
        return symfony
            .assets(asset_path + '/js/*.js', opt)
        ;
    };
    
    function coffee_stream(opt) {
        return symfony
            .assets(asset_path + '/coffee/*.coffee', opt)
                .pipe(coffeelint())
                .pipe(coffeelint.reporter('fail'))
                .pipe(coffee())
        ;
    };
    
    symfony.task('styles', () => {
        var options = {
            sourcemaps: true
        };
        
        return merge([
                css_stream(options),
                less_stream(options),
                sass_stream(options)
            ])
            
            .pipe(prefix({
                browsers: ['last 2 versions', 'ie 10'],
                cascade: false
            }))

            .pipe(cleancss())
            .pipe(concat(bundle_name + '.css'))
        ;
    });

    symfony.task('scripts', () => {
        var options = {
            sourcemaps: true
        };
        
        return merge ([
                js_stream(options),
                coffee_stream(options)
            ])
            
            .pipe(babel())

            .pipe(jshint())
            .pipe(jshint.reporter())

            .pipe(uglify())

            .pipe(concat(bundle_name + '.js'))
        ;
    });

    symfony.task('fonts', () => {
        return symfony
            .assets(asset_path + '/fonts/*.*')
        ;
    });

    symfony.task('images', () => {
        return symfony
            .assets(asset_path + '/images/*.*')
        ;
    });
    
    /*
     * These watch task should run the parent task rather than the child task.
     */
    symfony.task('watch:css', () => {
        symfony.watch([asset_path + '/{css, less, sass}/*.*'], ['css']);
    });
    
    symfony.task('watch:js', () => {
        symfony.watch([asset_path + '/{js, coffee}/*.*'], ['js']);
    });
    
    symfony.task('watch', () => {
        symfony.watch([asset_path + '/{css, less, sass}/*.*'], ['css']);
        symfony.watch([asset_path + '/{js, coffee}/*.*'], ['js']);
    });
};