'use strict';

var
    Path = require('path'),
    fs = require('fs'),
    json = require('jsonfile'),
    through = require('through2')
;

function Tracker(web_dir, path)
{
    if (!(this instanceof Tracker)) {
        return new Tracker(path);
    }
    
    if (typeof path !== 'string') {
        throw new Error('Expected a path.');
    }
    
    this.web = Path.resolve(web_dir);
    this.path = Path.resolve(path);
    this.cache = null;
    this.changed = false;
};

Tracker.prototype.load = function(cb)
{
    if (this.cache) {
        cb();
    }
    
    json.readFile(this.path, {throws: false}, (err, data) => {
        if (!err || err.code === 'ENOENT') {
            this.cache = data || {};
            return cb();
        }

        cb(err);
    });
}

Tracker.prototype.dump = function(cb)
{
    if (!this.cache || !this.changed) {
        return cb();
    }
    
    json.writeFile(this.path, this.cache, {spaces: 4}, (err) => {
        if (!err) {
            this.changed = false;
        }
        cb(err);
    });
};

Tracker.prototype.clear = function(cb)
{
    fs.unlink(this.path, (err) => {
        if (!err || err.code === 'ENOENT') {
            this.cache = null;
            this.changed = false;
        }
        cb(err);
    });
};

Tracker.prototype.add = function(task, file, cb)
{
    this.load((err) => {
        if (err) {
            return cb(err);
        }
        
        var rel = Path.relative(this.web, file);
        var base = Path.basename(file, Path.extname(file));
        
        if (!this.cache[task]) {
            this.cache[task] = {};
        }
        
        if (this.cache[task][base] && this.cache[task][base] !== rel) {
            throw new Error('Already tracking file ' + file);
        }
        
        this.cache[task][base] = rel;
        this.changed = true;
        
        cb();
    });
};

Tracker.prototype.track = function(task)
{
    var self = this;
    
    return through.obj((file, enc, cb) => {
        self.add(task, file.path, (err) => {
            cb(err, file);
        });
    });
};

module.exports = Tracker;